package com.biganto.visual.roompark.presentation.screen.albums

import android.content.Context
import android.graphics.drawable.Drawable
import android.os.Bundle
import android.view.View
import androidx.core.os.bundleOf
import androidx.core.widget.NestedScrollView
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.biganto.visual.roompark.conductor.dialogs.change_handler.DialogChangeHandler
import com.biganto.visual.roompark.domain.model.AlbumPreviewModel
import com.biganto.visual.roompark.domain.model.PhotoModel
import com.biganto.visual.roompark.presentation.screen.albums.util.AlbumListAdapter
import com.biganto.visual.roompark.presentation.screen.albums.util.AlbumsHeaderAdapter
import com.biganto.visual.roompark.presentation.screen.photo.PhotoScreenController
import com.biganto.visual.roompark.util.view_utils.grid.CeilsDecoration
import com.bluelinelabs.conductor.RouterTransaction
import com.bumptech.glide.Glide
import com.bumptech.glide.request.target.SimpleTarget
import com.bumptech.glide.request.target.Target
import com.bumptech.glide.request.transition.Transition
import com.google.android.material.textview.MaterialTextView
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import jp.wasabeef.glide.transformations.BlurTransformation
import jp.wasabeef.glide.transformations.ColorFilterTransformation
import java.util.concurrent.TimeUnit
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

const val SELECTED_ALBUM_INDEX_KEY = "SELECTED_ALBUM_INDEX"

class AlbumsScreenController :
    BigantoBaseController<AlbumsScreenViewState
            , AlbumsScreen
            , AlbumsScreenPresenter>
   , AlbumsScreen {

    constructor(args: Bundle):super(args)

    constructor(id: Int) : super(bundleOf(SELECTED_ALBUM_INDEX_KEY to id))


    @BindView(R.id.headers_recycler_view)
    lateinit var headersRecyclerView: RecyclerView

    @BindView(R.id.albums_recycler_view)
    lateinit var albumsRecyclerView: RecyclerView

    @BindView(R.id.photo_albums_container)
    lateinit var nestedScrollView: NestedScrollView


    @BindView(R.id.header_album_title)
    lateinit var currentAlbomTitle: MaterialTextView



//
//    private val photosBackgroundTarget = object : com.squareup.picasso.Target {
//        override fun onPrepareLoad(placeHolderDrawable: Drawable?) {
////            nestedScrollView.background = placeHolderDrawable
//        }
//
//        override fun onBitmapFailed(e: Exception?, errorDrawable: Drawable?) {
//            Timber.e(e)
//        }
//
//        override fun onBitmapLoaded(bitmap: Bitmap?, from: Picasso.LoadedFrom?) {
//            bitmap?.scaleCenterCrop(nestedScrollView)?.let {
//                nestedScrollView.background = BitmapDrawable(activity?.resources, it)
//            }
//        }
//    }


    private fun bindRecycler() {
        headersRecyclerView.isNestedScrollingEnabled = false
        headersRecyclerView.layoutManager =
            LinearLayoutManager(activity, RecyclerView.HORIZONTAL, false)
        headersRecyclerView.adapter = AlbumsHeaderAdapter()
        headersRecyclerView.itemAnimator = null

        if (headersRecyclerView.itemDecorationCount == 0)
            headersRecyclerView.addItemDecoration(
                CeilsDecoration(1
                    , resources?.getDimensionPixelSize(R.dimen.ceil_grid_padding))
            )

        albumsRecyclerView.isNestedScrollingEnabled = false
        albumsRecyclerView.layoutManager =
            LinearLayoutManager(activity, RecyclerView.VERTICAL, false)
        albumsRecyclerView.adapter = AlbumListAdapter()
        albumsRecyclerView.itemAnimator = null
    }

    override fun onViewBound(v: View) {
        toolBar.hideAll()
        bottomNavigationController.hide()
        bindRecycler()
    }

    override fun injectDependencies() {
        getComponent()
    }

    @Inject
    override lateinit var injectedPresenter: AlbumsScreenPresenter


//    @Inject
//    lateinit var snacky:ISnackBarProvider

    lateinit var rpActivity: RoomParkMainActivity

    fun getComponent() = DaggerAlbumsScreenComponent.factory()
        .create(RoomParkApplication.component
            ,activity as RoomParkMainActivity
            ,args.getInt(SELECTED_ALBUM_INDEX_KEY))
        .inject(this)

//    @Inject
//    lateinit var ac: RoomParkMainActivity


    override fun render(viewState: AlbumsScreenViewState) {
        when(viewState){
            is AlbumsScreenViewState.Idle -> render(viewState)
            is AlbumsScreenViewState.AlbumsSelected -> render(viewState)
            is AlbumsScreenViewState.AlbumsListLoaded-> render(viewState)
            is AlbumsScreenViewState.HeaderAlbumChoosed-> render(viewState)
            is AlbumsScreenViewState.PhotoSelected -> render(viewState)
            is AlbumsScreenViewState.SomeError -> render(viewState)
            is AlbumsScreenViewState.RestoreView -> render(viewState)
        }
    }

    private fun render(viewState: AlbumsScreenViewState.Idle){

    }

    private fun render(viewState: AlbumsScreenViewState.PhotoSelected){
        router.pushController(RouterTransaction.with(PhotoScreenController(viewState.photoId))
            .pushChangeHandler(DialogChangeHandler())
            .popChangeHandler(DialogChangeHandler())

        )
    }

    private fun render(viewState: AlbumsScreenViewState.SomeError) =
        showError(viewState.exception)

    private fun render(viewState: AlbumsScreenViewState.AlbumsListLoaded) {
        (headersRecyclerView.adapter as AlbumsHeaderAdapter).setItems(
            viewState.list.asSequence().sortedByDescending { it.published }.toList()
        )

        headersRecyclerView.let {
            it.scrollToPosition(
                (it.adapter as AlbumsHeaderAdapter).getItemPosition(viewState.selectedAlbumId)
            )
        }

        viewState.list.first { it.albumId == viewState.selectedAlbumId }.let {
            currentAlbomTitle.text = it.title
            activity?.let { ctx ->
                loadGlideBlurred(it.previewUrl, ctx, nestedBgTarget)
            }
        }
    }

    private fun render(viewState: AlbumsScreenViewState.HeaderAlbumChoosed) {
        (headersRecyclerView.adapter as AlbumsHeaderAdapter)
            .getItemPosition(viewState.item.albumId).let {
            headersRecyclerView.scrollToPosition(it)
        }

        currentAlbomTitle.text = viewState.item.title
        activity?.let { ctx ->
            loadGlideBlurred(viewState.item.previewUrl, ctx, nestedBgTarget)
        }

    }

    private fun render(viewState: AlbumsScreenViewState.AlbumsSelected){
        (albumsRecyclerView.adapter as AlbumListAdapter).setItems(
            viewState.list.asSequence().sortedByDescending { it.published }.toList()
        )
    }

    private fun render(viewState: AlbumsScreenViewState.RestoreView) {
        (albumsRecyclerView.adapter as AlbumListAdapter).setItems(
            viewState.restore.albumList.asSequence().sortedByDescending { it.published }.toList()
        )

        (headersRecyclerView.adapter as AlbumsHeaderAdapter).setItems(
            viewState.restore.albumsPreview.asSequence().sortedByDescending { it.published }.toList()
        )

        headersRecyclerView.let {
            it.scrollToPosition(
                (it.adapter as AlbumsHeaderAdapter).getItemPosition(viewState.restore.currentIndex)
            )
        }

        viewState.restore.albumsPreview.first { it.albumId == viewState.restore.currentIndex }
            .let {
                currentAlbomTitle.text = it.title
                activity?.let { ctx ->
                    loadGlideBlurred(it.previewUrl, ctx, nestedBgTarget)
                }
            }
    }

    private val nestedBgTarget  = object : SimpleTarget<Drawable>(){
        override fun onResourceReady(resource: Drawable, transition: Transition<in Drawable>?) {
            nestedScrollView.background = resource
        }
    }

    private fun loadGlideBlurred(url:String, context: Context, drawable: Target<Drawable>) =
        Glide.with(context)
        .load(url)
        .transform(BlurTransformation(13, 2))
        .transform(ColorFilterTransformation(0xCC000000.toInt()))
        .into(drawable)

    override fun onAlbumSelected(): Observable<AlbumPreviewModel>
            = (headersRecyclerView.adapter as AlbumsHeaderAdapter)
        .onItemClicked
        .map { it }
        .debounce (300L,TimeUnit.MILLISECONDS)
        .observeOn(AndroidSchedulers.mainThread())
        .doOnNext { (albumsRecyclerView.adapter as AlbumListAdapter).setItems(arrayListOf())}


    override fun onPhotoSelected(): Observable<PhotoModel>
            = (albumsRecyclerView.adapter as AlbumListAdapter)
        .photoObs
        .map { it }
        .debounce (300L,TimeUnit.MILLISECONDS)
        .observeOn(AndroidSchedulers.mainThread())




    override fun getLayoutId(): Int = R.layout.albums_screen

}