package com.biganto.visual.roompark.presentation.screen.article.util

import android.text.Html
import android.text.method.LinkMovementMethod
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import androidx.recyclerview.widget.RecyclerView
import butterknife.ButterKnife
import com.biganto.visual.roompark.R
import com.bumptech.glide.Glide
import com.google.android.material.textview.MaterialTextView

/**
 * Created by Vladislav Bogdashkin on 29.01.2020.
 */



sealed class HtmlTag {
    class Text(val text:String):HtmlTag()
    class ImageSource(val src:String):HtmlTag()
}

class HtmlPageAdapter : RecyclerView.Adapter<HtmlTagViewHolder<HtmlTag>>() {

    private val list = mutableListOf<HtmlTag>()

    override fun getItemViewType(position: Int): Int {
        return when (list[position]){
            is HtmlTag.Text -> HtmlViewType.TEXT.viewType
            is HtmlTag.ImageSource -> HtmlViewType.IMG.viewType
        }
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): HtmlTagViewHolder<HtmlTag> {
      return when (HtmlViewType.fromInt(viewType)) {
            HtmlViewType.TEXT ->
                HtmlTextViewHolder(
                    LayoutInflater.from(parent.context)
                        .inflate(R.layout.htlm_text_viewholder, parent, false)
                ) as HtmlTagViewHolder<HtmlTag>
            HtmlViewType.IMG ->
                HtmlImageViewHolder(
                    LayoutInflater.from(parent.context)
                        .inflate(R.layout.htlm_image_viewholder, parent, false)
                ) as HtmlTagViewHolder<HtmlTag>
        }
    }

    override fun onBindViewHolder(holder: HtmlTagViewHolder<HtmlTag>, position: Int) {
        holder.bindModel(list[position])
    }

    override fun getItemCount(): Int = list.size


    fun setItems(items:List<HtmlTag>){
        this.list.clear()
        this.list.addAll(items)
        notifyDataSetChanged()
    }
}

abstract class HtmlTagViewHolder<M:HtmlTag>(itemView: View) : RecyclerView.ViewHolder(itemView){

    abstract fun onViewBound(model: M)

    fun bindModel(model: M){
        ButterKnife.bind(this, itemView)
        bindedModel = model
        onViewBound(model)
    }

    protected lateinit var bindedModel: M
}

class HtmlTextViewHolder(itemView: View) : HtmlTagViewHolder<HtmlTag.Text>(itemView) {

    override fun onViewBound(model: HtmlTag.Text) {
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.N) {
            (itemView as MaterialTextView).text = Html.fromHtml(model.text,Html.FROM_HTML_MODE_COMPACT)
        } else {
            (itemView as MaterialTextView).text = Html.fromHtml(model.text)
        }
        itemView.movementMethod = LinkMovementMethod.getInstance()
    }
}

class HtmlImageViewHolder(itemView: View) :HtmlTagViewHolder<HtmlTag.ImageSource>(itemView) {
    override fun onViewBound(model: HtmlTag.ImageSource) {
        val url = "https://room-park.ru${model.src}"

        Glide.with(itemView)
            .load(url)
            .into(itemView as ImageView)
    }
}

private enum class HtmlViewType(val viewType:Int){

    TEXT(0),
    IMG(1);

    companion object {
        fun fromInt(type: Int) = HtmlViewType.values().first { it.viewType == type }
    }


}
