package com.biganto.visual.roompark.presentation.screen.deals

import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.DealsInteractor
import com.biganto.visual.roompark.domain.model.DealPreviewModel
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */


class DealsScreenPresenter @Inject constructor(
    private val interactor: DealsInteractor
)
    : BigantoBasePresenter<DealsScreen, DealsScreenViewState>() {

    override fun defaultErrorViewStateHandler() =
        {e:ExceptionString -> DealsScreenViewState.SomeError(e)}

    override fun bindIntents() {

        val getStatusList = interactor.getStatusList()

        val fetchDeals = interactor.fetchDeals()
            .flatMap { deals ->
                getStatusList
                    .map { List(deals.size) { index ->
                        DealPreviewModel(
                            Pair(deals[index], it)
                        )
                    } }
            }
            .map(DealsScreenViewState::DealsLoaded)

        val state = restoreStateObservable
            .mergeWith(fetchDeals)
            .doOnError{ Timber.e(it)}
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())

        subscribeViewState(state.cast(DealsScreenViewState::class.java), DealsScreen::render)
    }

}