package com.biganto.visual.roompark.presentation.screen.favorites

import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.FavoritesInteractor
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */


class FavoritesScreenPresenter @Inject constructor(
    private val interactor: FavoritesInteractor
)
    : BigantoBasePresenter<FavoritesScreen, FavoritesScreenViewState>() {

    override fun defaultErrorViewStateHandler() =
        {e: ExceptionString -> FavoritesScreenViewState.SomeError(e)}

    override fun bindIntents() {

        val prefetchCards = interactor.getFavoritesForCurrentUser()
            .map { FavoritesScreenViewState.FavoriteEstatesLoaded(it) }

        val state = restoreStateObservable
            .mergeWith(prefetchCards)
            .doOnError{ Timber.e(it)}
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())

        subscribeViewState(state.cast(FavoritesScreenViewState::class.java), FavoritesScreen::render)
    }

}