package com.biganto.visual.roompark.presentation.screen.feed_list

import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.ArticlesInteractor
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject
import javax.inject.Named

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */


class ArticlesScreenPresenter @Inject constructor(
    @Named(FEED_ALIAS) private val feedId:String,
    private val interactor: ArticlesInteractor
)
    : BigantoBasePresenter<ArticlesScreen, ArticlesScreenViewState>() {

    override fun defaultErrorViewStateHandler() =
        {e: ExceptionString -> ArticlesScreenViewState.SomeError(e)}

    override fun bindIntents() {
        Timber.d("feedId : $feedId")

        val prefetchCards = interactor.fetchArticles(feedId)
            .map<ArticlesScreenViewState>{ ArticlesScreenViewState.ArticlesLoaded(it.articles) }
            .startWith(Observable.just<ArticlesScreenViewState>(ArticlesScreenViewState.Idle()))

        val state = restoreStateObservable
            .mergeWith(prefetchCards)
            .doOnError{ Timber.e(it)}
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())

        subscribeViewState(state.cast(ArticlesScreenViewState::class.java), ArticlesScreen::render)
    }

}