package com.biganto.visual.roompark.presentation.screen.settings

import android.content.Context
import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.SettingsInteractor
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */


class SettingsScreenPresenter @Inject constructor(
    private val interactor: SettingsInteractor,
    private val activity: Context
)
    : BigantoBasePresenter<SettingsScreen, SettingsScreenViewState>() {

    override fun defaultErrorViewStateHandler() =
        {e: ExceptionString -> SettingsScreenViewState.SomeError(e)}

    override fun bindIntents() {

        val fetchSettings = interactor.fetchSettings()
            .flatMap {settings ->
                interactor.getCacheInfo()
                    .doOnNext { cached -> cached.sortBy { it.id } }
                    .doOnNext{ settings.cachedData = it }
                    .map { settings }}
            .map { SettingsScreenViewState.LoadSettingsList(it)}


        val onSignOut = intent(SettingsScreen::signOut)
            .flatMap {
                interactor.signOut()
                    .andThen(Observable.just<SettingsScreenViewState>(SettingsScreenViewState.SignOut()))
                    .onErrorReturn(::parseError)
            }


        val state = restoreStateObservable
            .mergeWith(fetchSettings)
            .mergeWith(onSignOut)
            .doOnError{ Timber.e(it)}
            .subscribeOn(Schedulers.io())
            .onErrorReturn(::parseError)
            .observeOn(AndroidSchedulers.mainThread())

        subscribeViewState(state.cast(SettingsScreenViewState::class.java), SettingsScreen::render)
    }

}